#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <limits>

using namespace std;
using ll = long long;
using ld = long double;

const ll infinity = numeric_limits<ll>::max() / 4;

struct Point {
    ll x{};
    ll y{};

    Point() {}
    Point(ll x, ll y) : x(x), y(y) {}

    friend istream& operator>>(istream& stream, Point& point) {
        return stream >> point.x >> point.y;
    }
};

struct Shortcut {
    Point start{};
    Point end{};

    Shortcut() {}
    Shortcut(Point start, Point end) : start(start), end(end) {}

    friend istream& operator>>(istream& stream, Shortcut& shortcut) {
        return stream >> shortcut.start >> shortcut.end;
    }
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, p;
    cin >> n >> p;

    vector<Shortcut> shortcuts(p);
    vector<ll> shortcut_distances(p, infinity);
    map<ll, map<ll, int>> by_start{};
    map<ll, map<ll, int>> by_end{};

    for (int i = 0; i < p; i++) {
        cin >> shortcuts[i];
        by_start[shortcuts[i].start.x][shortcuts[i].start.y] = i;
        by_end[shortcuts[i].end.x][shortcuts[i].end.y] = i;
    }

    by_start[n][n] = shortcuts.size();
    by_end[n][n] = shortcuts.size();

    shortcuts.push_back(Shortcut(Point(n, n), Point(n, n)));
    shortcut_distances.push_back(infinity);

    map<ll, int> optimal_by_y{};

    optimal_by_y[0] = shortcuts.size();
    shortcuts.push_back(Shortcut(Point(0, 0), Point(0, 0)));
    shortcut_distances.push_back(0);

    auto maybe_add_optimal = [&](ll x, ll y, int shortcut_index) {
        cerr << "by_end x " << x << " y " << y << "\n";
        const Shortcut& shortcut = shortcuts[shortcut_index];

        auto optimal_iter = optimal_by_y.lower_bound(y);

        if (optimal_iter == optimal_by_y.end() || optimal_iter->first > y) {
            optimal_iter = prev(optimal_iter);
        }

        int current_optimal_index = optimal_iter->second;
        const Shortcut& current_optimal = shortcuts[current_optimal_index];

        if (
            shortcut_distances[current_optimal_index] - current_optimal.end.x - current_optimal.end.y
                <= shortcut_distances[shortcut_index] - shortcut.end.x - shortcut.end.y
        ) {
            cerr << "throw out\n";
            return;
        }

        cerr << "accept\n";

        vector<ll> to_erase{};

        for (auto iter = optimal_by_y.lower_bound(y); iter != optimal_by_y.end(); iter++) {
            int existing_index = iter->second;
            const Shortcut& existing_shortcut = shortcuts[existing_index];

            if (
                shortcut_distances[shortcut_index] - shortcut.end.x - shortcut.end.y
                    <= shortcut_distances[existing_index] - existing_shortcut.end.x - existing_shortcut.end.y
            ) {
                cerr << "throw out " << existing_shortcut.end.x << " " << existing_shortcut.end.y << "\n";
                to_erase.push_back(iter->first);
            }
        }

        for (int y_to_erase : to_erase) {
            optimal_by_y.erase(y_to_erase);
        }

        optimal_by_y[y] = shortcut_index;
    };

    ll last_processed_by_end_x = -1;

    for (auto& [x, x_indexes] : by_start) {
        if (x < 0 || x > n) {
            continue;
        }

        cerr << "by_start x " << x << "\n";

        auto by_end_iter = by_end.upper_bound(last_processed_by_end_x);

        for (; by_end_iter != by_end.end(); by_end_iter++) {
            if (by_end_iter->first > x) {
                continue;
            }

            for (auto& [y, shortcut_index] : by_end_iter->second) {
                if (y < 0 || y > n || (shortcuts[shortcut_index].start.x == shortcuts[shortcut_index].end.x && by_end_iter->first == x)) {
                    continue;
                }
    
                maybe_add_optimal(x, y, shortcut_index);
            }
        }

        last_processed_by_end_x = by_end_iter == by_end.begin() ? -1 : prev(by_end_iter)->first;

        for (auto& [y, shortcut_index] : x_indexes) {
            if (y < 0 || y > n) {
                continue;
            }

            cerr << "by_start x " << x << " y " << y << "\n";

            auto optimal_iter = optimal_by_y.lower_bound(y);

            if (optimal_iter == optimal_by_y.end() || optimal_iter->first > y) {
                optimal_iter = prev(optimal_iter);
            }

            ll distance = 0;
            int optimal_index = optimal_iter->second;
            const Shortcut& optimal_shortcut = shortcuts[optimal_index];

            cerr << "optimal " << optimal_shortcut.end.x << " " << optimal_shortcut.end.y << "\n";

            if (optimal_index == -1) {
                distance = x + y;
            } else {
                distance = shortcut_distances[optimal_index] + x + y - optimal_shortcut.end.x - optimal_shortcut.end.y;
            }

            cerr << "distance " << distance << "\n";

            shortcut_distances[shortcut_index] = distance;
        }

        for (auto& [y, shortcut_index] : by_end[x]) {
            if (y < 0 || y > n || shortcuts[shortcut_index].start.x != shortcuts[shortcut_index].end.x) {
                continue;
            }

            maybe_add_optimal(x, y, shortcut_index);
        }
    }

    cout << shortcut_distances[shortcuts.size() - 2] << "\n";

    return 0;
}
