#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <queue>

using namespace std;
using ll = long long;
using ld = long double;

const ll infinity = numeric_limits<ll>::max() / 4;

struct Point {
    ll x{};
    ll y{};

    Point() {}
    Point(ll x, ll y) : x(x), y(y) {}

    friend istream& operator>>(istream& stream, Point& point) {
        return stream >> point.x >> point.y;
    }
};

struct Shortcut {
    Point start{};
    Point end{};
    ll distance { infinity };

    Shortcut() {}
    Shortcut(Point start, Point end) : start(start), end(end) {}

    friend istream& operator>>(istream& stream, Shortcut& shortcut) {
        return stream >> shortcut.start >> shortcut.end;
    }
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, p;
    cin >> n >> p;

    map<pair<ll, ll>, vector<pair<int, Point>>> shortcuts{};
    vector<Point> pois{};

    pois.push_back(Point(0, 0));
    pois.push_back(Point(n, n));

    for (int i = 0; i < p; i++) {
        Point start, end;
        cin >> start >> end;
        pois.push_back(start);
        pois.push_back(end);
        shortcuts[{ start.x, start.y }].push_back({ pois.size() - 1, end });
    }

    vector<ll> poi_distances(pois.size(), infinity);
    poi_distances[0] = 0;

    priority_queue<pair<ll, int>, vector<pair<ll, int>>, greater<pair<ll, int>>> plan{};
    plan.push({ 0, 0 });

    while (!plan.empty()) {
        auto [dist, curr_idx] = plan.top();
        plan.pop();

        if (poi_distances[curr_idx] != dist) {
            continue;
        }

        const Point& curr = pois[curr_idx];

        for (int i = 0; i < pois.size(); i++) {
            const Point& poi = pois[i];

            if (poi.x < curr.x || poi.y < curr.y) {
                continue;
            }

            ll new_dist = dist + poi.x + poi.y - curr.x - curr.y;

            if (new_dist < poi_distances[i]) {
                plan.push({ new_dist, i });
                poi_distances[i] = new_dist;
            }
        }

        for (auto [i, poi] : shortcuts[{ curr.x, curr.y }]) {
            if (poi.x < curr.x || poi.y < curr.y) {
                continue;
            }

            ll new_dist = dist;

            if (new_dist < poi_distances[i]) {
                plan.push({ new_dist, i });
                poi_distances[i] = new_dist;
            }
        }
    }

    cout << poi_distances[1] << "\n";

    return 0;
}
