#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>

using namespace std;
using ll = long long;
using ld = long double;

const ll mod = 1000000007;

struct DSU {
    int size;
    vector<int> dsu{};
    vector<int> component_levels{};
    vector<ll> ways_to_flood_children{};

    DSU(int size) {
        dsu.assign(size, -1);
        component_levels.resize(size);
        ways_to_flood_children.assign(size, 0);
    }

    int find(int node) {
        if (dsu[node] < 0) {
            return node;
        }

        return dsu[node] = find(dsu[node]);
    }

    void join(int parent, int child) {
        int leader1 = find(parent);
        int leader2 = find(child);
        int orig_leader1 = leader1;

        if (leader1 == leader2) {
            return;
        }

        int rank1 = -dsu[leader1];
        int rank2 = -dsu[leader2];

        if (rank1 < rank2) {
            swap(leader1, leader2);
        } else if (rank1 == rank2) {
            dsu[leader1] -= 1;
        }

        int result_leader = leader1;
        int other_leader = leader2;

        if (component_levels[leader1] != component_levels[leader2]) {
            if (component_levels[leader1] > component_levels[leader2]) {
                swap(leader1, leader2);
            }

            ways_to_flood_children[result_leader] = (ways_to_flood_children[leader1] * (ways_to_flood_children[leader2] + 1)) % mod;
        } else {
            ways_to_flood_children[result_leader] = max(ways_to_flood_children[result_leader], ways_to_flood_children[other_leader]);
        }

        dsu[other_leader] = result_leader;
        component_levels[result_leader] = min(component_levels[result_leader], component_levels[other_leader]);
    }
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n, m;
    cin >> n >> m;

    vector<vector<char>> map(n, vector<char>(m));

    for (auto& row : map) {
        for (auto& tile : row) {
            cin >> tile;
        }
    }

    DSU dsu(n * m);

    for (int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++) {
            dsu.component_levels[i * m + j] = i;
        }
    }

    for (int i = n - 2; i >= 1; i--) {
        for (int j = 1; j <= m - 2; j++) {
            if (map[i][j] == '#') {
                continue;
            }

            dsu.ways_to_flood_children[i * m + j] = 1;

            if (map[i][j - 1] == '.') {
                dsu.join(i * m + j - 1, i * m + j);
            }

            if (map[i + 1][j] == '.') {
                dsu.join(i * m + j, (i + 1) * m + j);
            }
        }

        vector<bool> seen_leaders(n * m, false);

        // cout << "--- " << i << " ---\n";

        // for (int k = 0; k < n * m; k++) {
        //     if (map[k / m][k % m] == '#') {
        //         continue;
        //     }

        //     int leader = dsu.find(k);

        //     if (seen_leaders[leader]) {
        //         continue;
        //     }

        //     seen_leaders[leader] = true;
        //     cout << leader << " " << dsu.ways_to_flood_children[leader] << "\n";
        // }
    }

    vector<bool> seen_leaders(n * m, false);
    ll result = 1;

    for (int i = 0; i < n * m; i++) {
        if (map[i / m][i % m] == '#') {
            continue;
        }

        int leader = dsu.find(i);

        if (seen_leaders[leader]) {
            continue;
        }

        seen_leaders[leader] = true;
        // cout << leader << " " << dsu.ways_to_flood_children[leader] << "\n";
        result = (result * (dsu.ways_to_flood_children[leader] + 1)) % mod;
    }

    cout << result << "\n";

    return 0;
}
